#!/usr/bin/perl
use strict;
use Test;
use Search::OpenFTS::Parser;
use IO::File;

BEGIN { plan tests =>1+219*4 }

my @list = (
{ t=>14, w=>'http://' },
{ t=>5, w=>'aew.werc.ewr/?ad=qwe&dw' },
{ t=>6, w=>'aew.werc.ewr' },
{ t=>18, w=>'/?ad=qwe&dw' },
{ t=>12, w=>' ' },
{ t=>5, w=>'1aew.werc.ewr/?ad=qwe&dw' },
{ t=>6, w=>'1aew.werc.ewr' },
{ t=>18, w=>'/?ad=qwe&dw' },
{ t=>12, w=>' ' },
{ t=>6, w=>'2aew.werc.ewr' },
{ t=>12, w=>' ' },
{ t=>14, w=>'http://' },
{ t=>5, w=>'3aew.werc.ewr/?ad=qwe&dw' },
{ t=>6, w=>'3aew.werc.ewr' },
{ t=>18, w=>'/?ad=qwe&dw' },
{ t=>12, w=>' ' },
{ t=>14, w=>'http://' },
{ t=>6, w=>'4aew.werc.ewr' },
{ t=>12, w=>' ' },
{ t=>14, w=>'http://' },
{ t=>5, w=>'5aew.werc.ewr:8100/?' },
{ t=>6, w=>'5aew.werc.ewr' },
{ t=>18, w=>':8100/?' },
{ t=>12, w=>'  ' },
{ t=>1, w=>'ad' },
{ t=>12, w=>'=' },
{ t=>1, w=>'qwe' },
{ t=>12, w=>'&' },
{ t=>1, w=>'dw' },
{ t=>12, w=>' ' },
{ t=>5, w=>'6aew.werc.ewr:8100/?ad=qwe&dw' },
{ t=>6, w=>'6aew.werc.ewr' },
{ t=>18, w=>':8100/?ad=qwe&dw' },
{ t=>12, w=>' ' },
{ t=>5, w=>'7aew.werc.ewr:8100/?ad=qwe&dw=%20%32' },
{ t=>6, w=>'7aew.werc.ewr' },
{ t=>18, w=>':8100/?ad=qwe&dw=%20%32' },
{ t=>12, w=>' ' },
{ t=>7, w=>'+4.0e-10' },
{ t=>12, w=>' ' },
{ t=>1, w=>'qwe' },
{ t=>12, w=>' ' },
{ t=>1, w=>'qwe' },
{ t=>12, w=>' ' },
{ t=>1, w=>'qwqwe' },
{ t=>12, w=>' ' },
{ t=>3, w=>'we' },
{ t=>12, w=>' ' },
{ t=>2, w=>'' },
{ t=>12, w=>'.' },
{ t=>2, w=>'' },
{ t=>12, w=>' ' },
{ t=>20, w=>'234.435' },
{ t=>12, w=>' ' },
{ t=>22, w=>'455' },
{ t=>12, w=>' ' },
{ t=>20, w=>'5.005' },
{ t=>12, w=>' ' },
{ t=>4, w=>'teodor@sigaev.ru' },
{ t=>12, w=>' ' },
{ t=>16, w=>'qwe-wer' },
{ t=>11, w=>'qwe' },
{ t=>12, w=>'-' },
{ t=>11, w=>'wer' },
{ t=>12, w=>' ' },
{ t=>17, w=>'-' },
{ t=>10, w=>'' },
{ t=>12, w=>'-' },
{ t=>10, w=>'' },
{ t=>12, w=>' ' },
{ t=>15, w=>'asdf-' },
{ t=>11, w=>'asdf' },
{ t=>12, w=>'-' },
{ t=>10, w=>'' },
{ t=>12, w=>' ' },
{ t=>3, w=>'asd' },
{ t=>12, w=>' ' },
{ t=>15, w=>'-238' },
{ t=>10, w=>'' },
{ t=>12, w=>'-' },
{ t=>22, w=>'238' },
{ t=>12, w=>' ' },
{ t=>15, w=>'1-' },
{ t=>22, w=>'1' },
{ t=>12, w=>'-' },
{ t=>10, w=>'' },
{ t=>12, w=>' ' },
{ t=>13, w=>' ' },
{ t=>1, w=>'qwer' },
{ t=>12, w=>' ' },
{ t=>1, w=>'jf' },
{ t=>12, w=>' ' },
{ t=>1, w=>'sdjk' },
{ t=>13, w=>' ' },
{ t=>12, w=>' ' },
{ t=>3, w=>'ewr1' },
{ t=>12, w=>'>' },
{ t=>12, w=>' ' },
{ t=>3, w=>'ewri2' },
{ t=>12, w=>' ' },
{ t=>13, w=>' ' },
{ t=>12, w=>"\n" },
{ t=>19, w=>'/usr/local/fff' },
{ t=>12, w=>' ' },
{ t=>19, w=>'/awdf/dwqe/4325' },
{ t=>12, w=>' ' },
{ t=>19, w=>'rewt/ewr' },
{ t=>12, w=>' ' },
{ t=>2, w=>'' },
{ t=>12, w=>'/' },
{ t=>3, w=>'234ac' },
{ t=>12, w=>' ' },
{ t=>1, w=>'wefjn' },
{ t=>12, w=>' ' },
{ t=>19, w=>'/wqe-324/ewr' },
{ t=>12, w=>' ' },
{ t=>6, w=>'gist.h' },
{ t=>12, w=>' ' },
{ t=>6, w=>'gist.h.c' },
{ t=>12, w=>' ' },
{ t=>6, w=>'gist.c' },
{ t=>12, w=>'.' },
{ t=>12, w=>' ' },
{ t=>1, w=>'readline' },
{ t=>12, w=>' ' },
{ t=>20, w=>'4.2' },
{ t=>12, w=>' ' },
{ t=>20, w=>'4.2' },
{ t=>12, w=>'.' },
{ t=>12, w=>' ' },
{ t=>20, w=>'4.2' },
{ t=>12, w=>',' },
{ t=>12, w=>' ' },
{ t=>15, w=>'readline-4' },
{ t=>11, w=>'readline' },
{ t=>12, w=>'-' },
{ t=>20, w=>'4.2' },
{ t=>12, w=>' ' },
{ t=>15, w=>'readline-4' },
{ t=>11, w=>'readline' },
{ t=>12, w=>'-' },
{ t=>20, w=>'4.2' },
{ t=>12, w=>'.' },
{ t=>12, w=>' ' },
{ t=>22, w=>'234' },
{ t=>12, w=>" \n" },
{ t=>13, w=>' ' },
{ t=>12, w=>' ' },
{ t=>1, w=>'wow' },
{ t=>12, w=>'  ' },
{ t=>12, w=>'<' },
{ t=>12, w=>' ' },
{ t=>1, w=>'jqw' },
{ t=>12, w=>' ' },
{ t=>12, w=>'<' },
{ t=>12, w=>'>' },
{ t=>12, w=>' ' },
{ t=>1, w=>'qwerty' },
{ t=>12, w=>'   ' },
{ t=>7, w=>'-2.34e-236' },
{ t=>12, w=>' ' },
{ t=>7, w=>'2e22' },
{ t=>12, w=>' ' },
{ t=>8, w=>'2.3.4' },
{ t=>3, w=>'e5' },
{ t=>12, w=>' ' },
{ t=>8, w=>'2.3.4' },
{ t=>12, w=>' ' },
{ t=>20, w=>'-2.3' },
{ t=>12, w=>' ' },
{ t=>20, w=>'+2.3' },
{ t=>12, w=>' ' },
{ t=>20, w=>'2.3' },
{ t=>12, w=>' ' },
{ t=>22, w=>'23' },
{ t=>12, w=>' ' },
{ t=>21, w=>'-23' },
{ t=>12, w=>' ' },
{ t=>21, w=>'+23' },
{ t=>12, w=>'  ' },
{ t=>3, w=>'23th' },
{ t=>12, w=>' ' },
{ t=>20, w=>'4.2' },
{ t=>12, w=>'-' },
{ t=>1, w=>'readline' },
{ t=>12, w=>' ' },
{ t=>15, w=>'4-readline' },
{ t=>22, w=>'4' },
{ t=>12, w=>'-' },
{ t=>11, w=>'readline' },
{ t=>12, w=>' ' },
{ t=>15, w=>'readline-4' },
{ t=>11, w=>'readline' },
{ t=>12, w=>'-' },
{ t=>22, w=>'4' },
{ t=>12, w=>' ' },
{ t=>12, w=>'&' },
{ t=>1, w=>'quot' },
{ t=>12, w=>' ' },
{ t=>23, w=>'&nbsp;' },
{ t=>12, w=>' ' },
{ t=>12, w=>'&' },
{ t=>12, w=>'#' },
{ t=>22, w=>'234' },
{ t=>12, w=>' ' },
{ t=>12, w=>'&' },
{ t=>12, w=>'#' },
{ t=>1, w=>'aqsd' },
{ t=>12, w=>';' },
{ t=>12, w=>' ' },
{ t=>23, w=>'&#22;' },
{ t=>12, w=>' ' },
{ t=>12, w=>'&' },
{ t=>12, w=>'#' },
{ t=>3, w=>'2dd' },
{ t=>12, w=>' ' },
{ t=>23, w=>'&#123;' },
{ t=>1, w=>'wr' },
{ t=>12, w=>" \n" },
);

my $p=Search::OpenFTS::Parser->new;
ok(ref $p, 'Search::OpenFTS::Parser' );

my $file = 't/datatest';

my $fh = IO::File->new("<$file") || die;
my $text;
$fh->read( $text, -s $file );
undef $fh;

$p->start_parser( \$text );
my ($type, $word, @bytxt );
while( ( ($type, $word) = $p->get_word ) && $type ) {
#print STDERR "{ t=>$type, w=>'$word' },\n";
	push( @bytxt, { t=>$type, w=>$word } );
}

$p->end_parser;

$fh = IO::File->new("<$file") || die;
$p->start_parser( $fh );

while( ( ($type, $word) = $p->get_word ) && $type ) {
	my $old = shift @bytxt;
	ok( $old->{t}, $type );
	ok( $old->{w}, $word );
	$old = shift @list;
	ok( $old->{t}, $type );
	ok( $old->{w}, $word );
}
$p->end_parser;


